# -*- coding: UTF-8 -*-

import xml.dom.minidom
import xml.etree.ElementTree as ET
import codecs
import datetime
import re
import os
import sys
from common import filesmodify, logger, macros

saleFiles = set()


def run(salesFunc, securityLogsFunc, shiftsFunc, cash, node, workDirectory, currentSales):
    global saleFiles
    log = logger.getUnloadLogger("Node", node.id)
    log.info(u"START. Cash №" + str(cash.code))

    convPath = node.currentConverter.onlineSalePath if currentSales else node.currentConverter.path
    currentPath = macros.applyMacros(convPath, log, shopcode=node.id, cashcode=cash.code)
    salePath = os.path.join(workDirectory, os.path.basename(currentPath))

    shifts = {}
    shopCode = 0

    for shift in shiftsFunc():
        timeBeg = shift['timeBeg'] if shift['timeBeg'] else datetime.datetime.now()
        shifts[shift['shift']] = {'timeBeg': timeBeg}
        if shift['shopcode']:
            shopCode = shift['shopcode']

    if not len(shifts):
        log.info(u'Нет смен для выгрузки')
        return

    level1 = ET.Element('Level1')
    tree = ET.ElementTree(level1)

    level2 = ET.SubElement(level1, 'Level2')

    level1Root = ET.SubElement(level1, 'ROOT')
    level1Root.text = node.currentConverter.rootString if sys.version_info.major == 3 else node.currentConverter.rootString.decode("utf-8")
    level2Shop = ET.SubElement(level2, 'SHOP')
    level2Shop.text = shopCode

    salePos = {}
    returnPos = {}
    payments = {}
    payments[1] = {}
    payments[2] = {}
    payments[25] = {}
    shiftAlreadyUnload = False

    for check in salesFunc():
        if check['docType'] not in [1, 2, 25]:
            continue

        if check['closed'] not in [1, 2]:
            continue

        if not shiftAlreadyUnload:
            level3 = ET.SubElement(level2, 'Level3')
            level3Date = ET.SubElement(level3, 'DATE')
            level3Date.text = shifts[check['shift']]['timeBeg'].strftime("%m/%d/%Y %H:%M:%S")
            shiftAlreadyUnload = True

        for pos in check['inventPositions']:
            if check['docType'] == 1:
                if pos['inventCode'] not in salePos:
                    salePos[pos['inventCode']] = [0, 0, 0, 0]
                    fillTaxInfo(salePos[pos['inventCode']], pos)
                salePos[pos['inventCode']][0] += pos['quant']
                salePos[pos['inventCode']][1] += pos['posSum']
            else:
                if pos['inventCode'] not in returnPos:
                    returnPos[pos['inventCode']] = [0, 0, 0, 0]
                    fillTaxInfo(returnPos[pos['inventCode']], pos)
                returnPos[pos['inventCode']][0] += pos['quant']
                returnPos[pos['inventCode']][1] += pos['posSum']

        log.info(u"смена №%s cashcode №%s document №%s " % (check['shift'], str(cash.code), str(check['docNum'])))

        for moneyPos in check['moneyPositions']:
            if moneyPos['valCode'] not in payments[check['docType']]:
                payments[check['docType']][moneyPos['valCode']] = 0
            if moneyPos['opCode'] in [70, 74]:
                payments[check['docType']][moneyPos['valCode']] += moneyPos['sumB']
            elif moneyPos['opCode'] in [72, 76]:
                payments[check['docType']][moneyPos['valCode']] -= moneyPos['sumB']

    level3Payments = ET.SubElement(level3, 'payments')
    addPayments(payments, level3Payments)

    level4 = ET.SubElement(level3, 'Level4')

    level4Sale = ET.SubElement(level4, 'SALE')
    level4Sale.text = "SALE"
    addPositions(salePos, level4)

    level4Return = ET.SubElement(level4, 'RETURN')
    level4Return.text = "RETURN"
    addPositions(returnPos, level4)

    pretty_xml = xml.dom.minidom.parseString(ET.tostring(tree.getroot(), encoding='utf-8')).toprettyxml()
    pretty_xml = removeRetsInTextTags(pretty_xml)
    with codecs.open(salePath, "w", "utf-8") as stream:
        stream.write("<?xml version=\"1.0\" encoding=\"UTF-8\"?>" + '\n')
        stream.write(re.sub(r'<\?.*\?>\n', '', pretty_xml))
        saleFiles.add(stream.name)

    statusDir = workDirectory
    statusFileName = 'ready'
    if currentSales:
        statusFileName += "Online"
        statusDir = os.path.dirname(workDirectory)

    converterSalesReadyFile = open('%s/%s' % (statusDir, statusFileName), 'w')
    converterSalesReadyFile.close()
    salesReadyFile = open('%s/%s' % (os.path.dirname(os.path.dirname(statusDir)), statusFileName), 'w')
    salesReadyFile.close()
    saleFiles.add(converterSalesReadyFile.name)
    saleFiles.add(salesReadyFile.name)


def fillTaxInfo(position, checkPosition):
    """
    Заполняем информацию о налогах: taxCode и taxRate
    position - позиция, для которой заполняется информация
    checkPosition - позиция в чеке, содержащая информацию о налогах
    """
    vatcode = 0
    vatrate = 0.0
    if checkPosition['vatcode5'] != 0:
        vatcode = checkPosition['vatcode5']
        vatrate = checkPosition['vatrate5']
    elif checkPosition['vatcode4'] != 0:
        vatcode = checkPosition['vatcode4']
        vatrate = checkPosition['vatrate4']
    elif checkPosition['vatcode3'] != 0:
        vatcode = checkPosition['vatcode3']
        vatrate = checkPosition['vatrate3']
    elif checkPosition['vatcode2'] != 0:
        vatcode = checkPosition['vatcode2']
        vatrate = checkPosition['vatrate2']
    elif checkPosition['vatcode1'] != 0:
        vatcode = checkPosition['vatcode1']
        vatrate = checkPosition['vatrate1']
    position[2] = vatcode
    position[3] = vatrate


def addPositions(positions, level4):
    """
    Добавляем в тег новую запись о товарах
    level4 - тег
    positions - список товаров
    """
    for pos in sorted(positions):
        level5 = ET.SubElement(level4, 'Level5')

        level5Code = ET.SubElement(level5, 'CODE')
        level5Code.text = pos

        level5Quantity = ET.SubElement(level5, 'QUANTITY')
        level5Quantity.text = str(positions[pos][0])

        level5Amount = ET.SubElement(level5, 'AMOUNT')
        level5Amount.text = str(positions[pos][1])

        level5TaxCode = ET.SubElement(level5, 'TAXCODE')
        level5TaxCode.text = str(positions[pos][2])

        level5TaxRate = ET.SubElement(level5, 'TAXRATE')
        level5TaxRate.text = str(positions[pos][3])


def addPayments(payments, level3Payments):
    """
    Добавляем в тег информацию об оплатах
    level3Payments - тег
    payments - список оплат
    """
    for type, paymentsByType in payments.items():
        for valcode, amount in paymentsByType.items():
            paymentTag = ET.SubElement(level3Payments, 'payment' if type == 1 else 'returnpayment')
            paymentValCode = ET.SubElement(paymentTag, 'valcode')
            paymentValCode.text = str(valcode)

            paymentAmount = ET.SubElement(paymentTag, 'amount')
            paymentAmount.text = str(amount)


def removeRetsInTextTags(xml):
    return re.sub(r'>\n\s*([^<>]*.[^<>\s]*)\n\s*</', '>\g<1></', xml)


def sendSales(cash, node, workDirectory, backupDir):
    log = logger.getUnloadLogger("Node", node.id)
    rewriteDestFile = filesmodify.checkFilesInRemoteDirectory(rewrite=node.currentConverter.rewriteDestFile,
            workDirectory=workDirectory, remoteDirectory=str(os.path.dirname(node.currentConverter.path)))
    return filesmodify.safeMoveDirectory(log, workDirectory, os.path.dirname(node.currentConverter.path),
            backupDir=backupDir, rewrite=rewriteDestFile, maxCountFiles=cash.maxCountFiles, actualCountDays=cash.actualCountDays)


def sendOnlineSales(cash, node, workDirectory):
    log = logger.getUnloadLogger("Node", node.id)
    return filesmodify.safeMoveDirectory(log, workDirectory, os.path.dirname(node.currentConverter.onlineSalePath))


def getSaleFiles():
    global saleFiles
    return saleFiles
